/* SPDX-FileCopyrightText: 2009-2023 Greenbone AG
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/**
 * @file
 * @brief UUID creation.
 */

#ifndef _GVM_UTIL_UUIDUTILS_H
#define _GVM_UTIL_UUIDUTILS_H

char *
gvm_uuid_make (void);

#endif /* not _GVM_UTIL_UUIDUTILS_H */
