/* SPDX-FileCopyrightText: 2021-2023 Greenbone AG
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef _GVM_BOREAS_ARP_H
#define _GVM_BOREAS_ARP_H

void
send_arp_v4 (const char *);

#endif /* not _GVM_BOREAS_ARP_H */
