/* SPDX-FileCopyrightText: 2009-2024 Greenbone AG
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/**
 * @file
 * @brief Headers for version utils.
 */

#ifndef _GVM_UTIL_VERSIONUTILS_H
#define _GVM_UTIL_VERSIONUTILS_H

#include <glib.h>
#include <stdio.h>

int
cmp_versions (const char *, const char *);

#endif /* not _GVM_UTIL_VERSIONUTILS_H */
