/* SPDX-FileCopyrightText: 2019-2023 Greenbone AG
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/**
 * @file
 * @brief Version utilities.
 */

#ifndef _GVM_BASE_VERSION_H
#define _GVM_BASE_VERSION_H

const char *
gvm_libs_version (void);

#endif /* not _GVM_BASE_VERSION_H */
